/*

    options.c - NetCrack Official Client 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include "client.h"
#include "options.h"

void usage(char *cmd) {
	
	printf("\nNetCrack Official Client %s - (C) 2009 Salvatore Fresta\n"
           "http://www.salvatorefresta.net\n"
           "\n"
           "Usage: %s [options]\n"
           "\n"
           "Required options:\n"
           "\n"
           "\t--host            : Server hostname or IP address\n"
           "\t--port            : Server TCP port\n"
           "\t--password        : Server authentication password (default: netcrack)\n"
           "\n"
           "Common options:\n"
           "\n"
#ifndef WIN32
#ifdef SAFE
           "\t--safe            : Enable the safe mode\n"
           "\t--display <value> : Optional display for safe mode (GNU/Linux)\n"
#endif
           "\t--daemon          : Start as daemon\n"
#endif
           "\t--verbose         : Enable the verbose mode\n"
           "\t--help            : Display this help\n\n",
           VERSION, cmd);
           
}

int vspr(char **buff, char *fmt, ...) {
	
    va_list ap;
    int     len,
            mlen;
    char    *ret = NULL;

    mlen = strlen(fmt) + 128;
    va_start(ap, fmt);
    
    while(1) {
    	
        ret = (char *) malloc(mlen);
        if(!ret) return -1;
        
        len = vsnprintf(ret, mlen, fmt, ap);
        
        if((len >= 0) && (len < mlen)) break;
        
        mlen = len < 0 ? mlen+128 : len+1;
        
        free(ret);
        
    }
    
    va_end(ap);

    *buff = ret;
    
    return len;
    
}

#ifndef WIN32

int daemonize(void) {
	
	pid_t pid;
	
	if((pid=fork()) < 0)
		return 0;
	else
		if(pid > 0) exit(0);
	
	setsid();
		
	return 1;
	
}

#endif
