/*

    socks.c - NetCrack Official Client 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#ifdef WIN32
	#include <winsock.h>
	#define close closesocket
#else
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <unistd.h>
	#include <errno.h>
	#include <netdb.h>
#endif
#include "socks.h"

int socket_connect(char *server, int port) {

	int sd;
	struct sockaddr_in sock;
	struct hostent *host = NULL;
	
#ifdef WIN32	
	WSADATA wsadata;
    if(WSAStartup(MAKEWORD(1,0), &wsadata)) return -1;
#endif
	
	memset(&sock, 0, sizeof(sock));
	
	if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0) return -1;
	
	sock.sin_family = AF_INET;
	sock.sin_port = htons(port);
	
	if(!(host=gethostbyname(server))) return -1;
	
	sock.sin_addr = *((struct in_addr *)host->h_addr);
	
	if(connect(sd, (struct sockaddr *) &sock, sizeof(sock)) < 0) return -1;
	
	return sd;
   
}

char *socket_read_pkg(int sd, int *error) {
	
    static char cache[140],
                *ret = NULL;
    static int cachesz = 0,
               retsz = 0;
    struct timeval timeout;
	fd_set input;
    int     i,
            t,
            eol,
            ret_len = 0;

    if(!sd) {
        cachesz = 0;
        return(NULL);
    }

redo:
    eol = -1;
    do {
        if(cachesz <= 0) {
        	
        	FD_ZERO(&input);
			FD_SET(sd, &input);
			
			timeout.tv_sec  = 1;
			timeout.tv_usec  = 0;
        	
			if(select(sd+1, &input, NULL, NULL, &timeout) <= 0) return NULL;
			
            cachesz = recv(sd, cache, sizeof(cache), 0);
            
            if(cachesz <= 0) {
                if(ret_len)
					eol = 0;
				else {
					*error = 1;
					if(ret) {
						free(ret);
						ret = NULL;
					}
					return NULL;
				}
                break;
            }
            
        }

        for(i = 0; i < cachesz; i++) {
            if((cache[i] == '\n')) {
            	if(eol < 0) eol = i+1;
            } else if(eol >= 0) {
                break;
            }
        }

        t = (eol < 0) ? cachesz : eol;
        if((ret_len + t) >= retsz) {
            retsz = ((ret_len + t + 1) + 0xff) & (~0xff);
            ret = realloc(ret, retsz);
            if(!ret) return(NULL);
        }
        memcpy(ret + ret_len, cache, t);
        ret_len += t;

        cachesz -= i;
        memmove(cache, cache + i, cachesz);
        
    } while(eol < 0);

    if(eol < 0) return(NULL);
    if(!ret_len) goto redo;
    ret[ret_len] = 0;
    
    return ret;
    
}

int socket_send(int socket, char *buffer, size_t size) {
	
	if(socket < 0) return -1;

	return send(socket, buffer, size, 0) < 0 ? -1 : 0;
	
}

int socket_close(int socket) {
	
	if(socket < 0) return -1;
	
	return close(socket) < 0 ? -1 : 0;
	
}
