/*

    main.c - NetCrack Official Client 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include "include/options.h"
#include "include/client.h"
#include "include/socks.h"
#include "include/pkg.h"

int main(int argc, char *argv[]) {
	
	int sock,
	    port = 0,
	    option = 0,
	    optidx = 0;
	
	extern int found, set_daemon, verbose;
#ifdef SAFE
	extern int safe;
#endif
	extern char *display;
	
	char *host     = NULL,
	     *password = "netcrack";
	
	struct option long_options[] = {
            {"host",             1, 0, 1},
            {"port",             1, 0, 2},
            {"password",         1, 0, 3},
            {"display",          1, 0, 4},
#ifdef SAFE
            {"safe",             0, 0, 's'},
#endif
            {"daemon",           0, 0, 'd'},
            {"verbose",          0, 0, 'v'},
            {"help",             0, 0, 'h'},
            {NULL,               0, 0, 0}
	};
	
	if(argc < 2) {
		usage(argv[0]);
        return -1;
	}
        
    while((option = getopt_long(argc, argv, "sdvh", long_options, &optidx)) > 0) {
		
		switch(option) {
			
			case 1:
				host = optarg;
				break;
				
			case 2:
				port = atoi(optarg);
				break;
				
			case 3:
				password = optarg;
				break;
				
			case 4:
				display = optarg;
				break;
			
			case 'd':
				set_daemon = 1;
				break;
				
			case 'v':
				verbose = 1;
				break;
#ifdef SAFE			
			case 's':
				safe = 1;
				break;
#endif	
			case 'h':
				usage(argv[0]);
				return -1;
			break;
			
			default:
				usage(argv[0]);
				return -1;
			break;
			
		}
		
    }
    
    if(!host) {
    	printf("\nPlease set a valid hostname.\n\n");
    	return -1;
    }
    
    if(port <= 0 || port > 65535) {
    	printf("\nPlease set a valid TCP port number.\n\n");
    	return -1;
    }
    
    if(!password) {
    	printf("\nPlease set a valid password.\n\n");
    	return -1;
    }
    
#ifndef WIN32
	if(set_daemon && verbose) {
		printf("\nYou can't choose verbose mode and daemon mode togheter.\n\n");
		return -1;
	}
	/* Daemon mode for Unix systems only */
	if(set_daemon) daemonize();
#endif

	sock = socket_connect(host, port);
	
	if(sock < 0) {
		if(verbose) perror("\nmain socket");
		return -1;
	}
	
	if(do_auth(sock, password) < 0 && verbose) {
		printf("\nAuthentication failed! Please check the password.\n\n");
		socket_close(sock);
		return -1;
	}
	
	listen_from_server(sock);
	
	socket_close(sock);
	
	if(found == 1 && verbose) printf("Password found!\n\n");
	
	return 0;
	
}