/*

    pkg.c - NetCrack Server 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "pkg.h"
#include "socks.h"

int send_ci_packet(int sock, char *hash, char *cset, unsigned int keylen, unsigned int keyblock, unsigned int hashid) {
	
	int pkglen,
	    csetlen,
	    len;
	char *pkg     = NULL,
	     *b64cset = NULL;
	
	if(!hash || !cset) return -1;
	
	csetlen = strlen(cset);
	b64cset = (char *)base64_encode((unsigned char *)cset, &csetlen);
	
	if(!b64cset) return -1;
	
	pkglen = strlen(PKG_CI)+strlen(hash)+strlen(b64cset)+11*3+7;
	
	pkg = (char *) calloc(pkglen, sizeof(char));
	
	if(!pkg) {
		perror("send_ci_packet calloc");
		return -1;
	}
	
	len = snprintf(pkg, pkglen, "%s %s %s %d-%d-%d\n", PKG_CI,
	                                                   b64cset,
	                                                   hash,
	                                                   keylen,
	                                                   keyblock,
	                                                   hashid);
												 
	if(len < 0 || len >= pkglen) {
		free(pkg);
		return -1;
	}
	                                           
	socket_send(sock, pkg, strlen(pkg));
	
	memset(pkg, 0, pkglen);
	free(pkg);
	
	return 0;
	
}

int send_data_packet(int sock, char *cset) {
	
	int pkglen,
	    len;
	char *pkg = NULL;
	
	if(!cset) return -1;
	
	pkglen = strlen(PKG_DT)+strlen(cset)+3;
	
	pkg = (char *) calloc(pkglen, sizeof(char));
	
	if(!pkg) {
		perror("send_data_packet calloc");
		return -1;
	}
	
	len = snprintf(pkg, pkglen, "%s %s\n", PKG_DT, cset);
	
	if(len < 0 || len >= pkglen) {
		free(pkg);
		return -1;
	}
	
	socket_send(sock, pkg, strlen(pkg));
	free(pkg);
	
	return 0;
	
}

int send_simple_packet(int sock, char *pkg) {
	
	if(!pkg) return -1;
	
	return socket_send(sock, pkg, strlen(pkg));
	
}

/* Base64 encoder by Luigi Auriemma */
unsigned char *base64_encode(unsigned char *data, int *size) {
    int     len,
            a,
            b,
            c;
    unsigned char      *buff,
            *p;
    static const unsigned char base[64] = {
        'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
        'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
        'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
        'w','x','y','z','0','1','2','3','4','5','6','7','8','9','+','/'
    };

    if(!size || (*size < 0)) {      // use size -1 for auto text size!
        len = strlen((char *)data);
    } else {
        len = *size;
    }
    buff = malloc(((len / 3) << 2) + 6);
    if(!buff) return(NULL);

    p = buff;
    do {
        a = data[0];
        b = data[1];
        c = data[2];
        *p++ = base[(a >> 2) & 63];
        *p++ = base[(((a &  3) << 4) | ((b >> 4) & 15)) & 63];
        *p++ = base[(((b & 15) << 2) | ((c >> 6) &  3)) & 63];
        *p++ = base[c & 63];
        data += 3;
        len  -= 3;
    } while(len > 0);
    for(*p = 0; len < 0; len++) *(p + len) = '=';

    if(size) *size = p - buff;
    return(buff);
}
