/*

    socks.c - NetCrack Server 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#ifdef WIN32
    #include <winsock.h>
    #define close   closesocket
#else
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <unistd.h>
	#include <errno.h>
	#include <netdb.h>
#endif
#include "socks.h"

int socket_connect(char *server, int port) {

	int sd;
	struct sockaddr_in sock;
	struct hostent *host;
	
	memset(&sock, 0, sizeof(sock));
	
	if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0) return -1;
	
	sock.sin_family = AF_INET;
	sock.sin_port = htons(port);
	
	if(!(host=gethostbyname(server))) return -1;
	
	sock.sin_addr = *((struct in_addr *)host->h_addr);
	
	if(connect(sd, (struct sockaddr *) &sock, sizeof(sock)) < 0) return -1;
	
	return sd;
   
}

char *socket_read_spkg(int sock, int *error) {

	int k = 0,
	    len = 0,
		count;
	char *buffer = NULL,
	     tmp;
	struct timeval timeout;
	fd_set input;
	
	while(1) {
	
		FD_ZERO(&input);
		FD_SET(sock, &input);
	
		timeout.tv_sec  = 1;
		timeout.tv_usec  = 0;
	
		if(select(sock+1, &input, NULL, NULL, &timeout) <= 0) {
			if(buffer) free(buffer);
			return NULL;
		}
		
		count = recv(sock, &tmp, 1, 0);
		
		if(count <= 0) {
			if(buffer) free(buffer);
			*error = 1;
			return NULL;
		}
		
		if(k >= len) {
			len += 80;
			
			buffer = (char *) realloc(buffer, len+1);
			
			if(!buffer) {
				perror("\nsocket_read_spkg realloc");
				return NULL;
			}
		}
		
		if(tmp != '\n')
			buffer[k++] = tmp;
		else {			
			if(!k) continue;
			buffer[k++] = tmp;
			buffer[k] = 0;
			break;	
		}	
	
	}
	
	return buffer;

}

int socket_send(int socket, char *buffer, size_t size) {
	
	if(socket < 0) return -1;

	return (send(socket, buffer, size, 0) != size) ? -1 : 0;
	
}

int socket_close(int socket) {
	
	if(socket < 0) return -1;
	
	return close(socket) < 0 ? -1 : 0;
	
}
