/*

    main.c - NetCrack Server 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include "include/server.h"
#include "include/options.h"

int main(int argc, char *argv[]) {

	int option = 0,
        optidx = 0,
        port = 0,
        verbose = 0,
        ret;

#ifdef SQL        
#ifndef WIN32
	int f_daemon = 0;
#endif
#endif

#ifndef SQL

	int keylen = 0,
        hashid = 0,
        hlen = 0,
        bchset = 0;

	char *hash = NULL,
         charset[16] = "azAZ09";
         
    extern Analys *info_data;
         
    verbose = 1;

#endif

	struct option long_options[] = {
				{"port",         1, 0, 'p'},
                {"list",         0, 0, 'l'},
				{"help",         0, 0, 'h'},
#ifdef SQL
#ifndef WIN32
                {"daemon",       0, 0, 'd'},
#endif
                {"verbose",      0, 0, 'v'},
#endif
#ifndef SQL
                {"hashid",       1, 0, 1},
                {"keylen",       1, 0, 2},
                {"hash",         1, 0, 3},
                {"c1",           0, 0, 4},
                {"c2",           0, 0, 5},
                {"c3",           0, 0, 6},
                {"c4",           0, 0, 7},
#endif
                {NULL,           0, 0, 0},
	};

	if(argc < 2) {
		usage(argv[0]);
		return -1;
	}

	while((option = getopt_long(argc, argv, "p:vldh", long_options, &optidx)) > 0) {

		switch(option) {

			case 'p':
				port = atoi(optarg);
			break;
			
#ifdef SQL
			case 'v':
				verbose = 1;
			break;

#ifndef WIN32
			
			case 'd':
				f_daemon = 1;
			break;
			
#endif
#endif
			case 'l':
				get_hash_id_list();
				return -1;
			break;

			case 'h':
				usage(argv[0]);
				return -1;
			break;

#ifndef SQL
			case 1:
				hashid = atoi(optarg);
			break;

			case 2:
				keylen = atoi(optarg);
			break;

			case 3:
				hash = optarg;
			break;

	        case 4:
               	if(!bchset) {
					memset(charset, 0, sizeof(charset));     
					bchset = 1;
				}
                if(!strstr(charset, "az")) strncat(charset, "az", sizeof(charset));
	        break;
                
            case 5:
            	if(!bchset) {
            		memset(charset, 0, sizeof(charset));
            		bchset = 1;
		    	}
            	if(!strstr(charset, "AZ")) strncat(charset, "AZ", sizeof(charset));
			break;
                
            case 6:
            	if(!bchset) {
            		memset(charset, 0, sizeof(charset));
            		bchset = 1;
		    	}
            	if(!strstr(charset, "09")) strncat(charset, "09", sizeof(charset));
	        break;
                
            case 7:
            	if(!bchset) {
            		memset(charset, 0, sizeof(charset));
            		bchset = 1;
            	}
		    	if(!strstr(charset, " /:@[`{~")) strncat(charset, " /:@[`{~", sizeof(charset));
		    break;

#endif

			default:
				usage(argv[0]);
				return -1;
			break;

		}

	}

	if(port <= 0 || port > 65535) {
		printf("\nPlease set a valid TCP port.\n\n");
		return -1;
	}

#ifndef SQL

	hlen = get_len_by_id(hashid)*2;

	if(!hash || strlen(hash) != hlen || check_hash_string(hash, hlen)) {
		printf("\nError: please set a valid hash.\n\n");
		return -1;
	}

	if(keylen <= 0) {
		printf("\nError: please set a valid key length.\n\n");
		return -1;
	}

	if(hashid < 0 || hashid > 1) {
		printf("\nError: please set a valid hash id.\n\n");
		return -1;
	}

	info_data = (Analys *) calloc(1, sizeof(Analys));

	if(!info_data) {
		if(verbose) perror("\nmain calloc");
		return -1;
	}

	strncpy(info_data->hash, hash, MAX_HASH_LENGTH*2+1);
	strncpy(info_data->charset, charset, sizeof(charset));
	info_data->hashid = hashid;
	info_data->keylen = keylen;
	
	/* Set a keyblock */
	if(info_data->keylen <= 5 && info_data->keylen > 1)
		info_data->keyblock = info_data->keylen-1;
	else
		if(info_data->keylen == 1)
			info_data->keyblock = 0;
		else
			info_data->keyblock = 5;

#endif

#ifdef SQL
#ifndef WIN32
	if(f_daemon && verbose) {
		printf("\nYou can't choose verbose mode and daemon mode togheter.\n\n");
		return -1;
	}
	/* Daemon mode for Unix systems only */
	if(f_daemon) daemonize();
#endif
#endif

	/* Start the server with max 10 clients in append */
	ret = start_server(port, 10, verbose);

#ifndef SQL
	if(verbose && ret >= 0) printf("Session complete.\n\n<-----\n\n");
#endif
	
	return 0;
	
}
